package main

import (
	"flag"
	"fmt"
	"net"
	"net/http"
	"os"

	"git.lattuga.net/boyska/circolog"
	syslog "gopkg.in/mcuadros/go-syslog.v2"
)

func main() {
	var err error
	syslogSocketPath := flag.String("syslogd-socket", "", "The socket to listen to syslog addresses")
	// dumpSocketPath := flag.String("dump-socket", "/run/buffer.sock", "The socket that user will connect to in order to receive logs")
	bufsize := flag.Int("buffer-size", 1000, "Number of messages to keep")
	syslogAddr := flag.String("syslog-addr", "127.0.0.1:9514", "Address:port where to listen for syslog messages")
	queryAddr := flag.String("query-addr", "127.0.0.1:9080", "Address:port where to bind the query service")
	querySocket := flag.String("query-socket", "", "Path to a unix domain socket for the HTTP server; recommended for security reasons!")
	flag.Parse()

	hub := circolog.NewHub(*bufsize)
	handler := syslog.NewChannelHandler(hub.LogMessages)

	server := syslog.NewServer()
	server.SetFormat(syslog.RFC5424)
	server.SetHandler(handler)
	if *syslogSocketPath != "" {
		if err = server.ListenUnixgram(*syslogSocketPath); err != nil {
			fmt.Fprintln(os.Stderr, "argh", err)
			os.Exit(1)
		}
		fmt.Printf("Binding socket `%s` [syslog]\n", *syslogSocketPath)
	} else {
		fmt.Printf("Binding address `%s` [syslog]\n", *syslogAddr)
		if err = server.ListenUDP(*syslogAddr); err != nil {
			fmt.Fprintln(os.Stderr, "argh", err)
			os.Exit(1)
		}
	}
	if err = server.Boot(); err != nil {
		fmt.Fprintln(os.Stderr, "argh", err)
		os.Exit(1)
	}
	go hub.Run()

	setupHTTP(hub)
	if *querySocket != "" {
		fmt.Printf("Binding address `%s` [http]\n", *querySocket)
		unixListener, err := net.Listen("unix", *querySocket)
		if err != nil {
			fmt.Fprintln(os.Stderr, "Error binding HTTP unix domain socket", err)
		}
		defer os.Remove(*querySocket)
		http.Serve(unixListener, nil)
	} else {
		fmt.Printf("Binding address `%s` [http]\n", *queryAddr)
		http.ListenAndServe(*queryAddr, nil)
	}
	server.Wait()
}
